﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Reflection;
using System.IO;
using System.Timers;
using SpeechEngineLibrary;
using System.Xml;
using System.Threading;
using System.Speech.Synthesis;
using System.Diagnostics;

namespace LabiHAL
{
    /// <summary>
    /// Interaction logic for HALWindow.xaml
    /// </summary>
    public partial class HALWindow : Window
    {
        public SpeechEngine Speech;
        public HALWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            #region Load Resourcers
            Assembly ThisAssembly = Assembly.GetExecutingAssembly();
            String NameSpace = this.GetType().Namespace;
            using (Stream Stream = ThisAssembly.GetManifestResourceStream(NameSpace+".Resources.PANEL_Fill.png"))
            {
                PlayControls.Background = new ImageBrush(BitmapFrame.Create(Stream));
            }

            using (Stream Stream = ThisAssembly.GetManifestResourceStream(NameSpace+".Resources.PANEL_Left.png"))
            {
                PanelLeft.Source = BitmapFrame.Create(Stream);
            }

            using (Stream Stream = ThisAssembly.GetManifestResourceStream(NameSpace+".Resources.PANEL_Right.png"))
            {
                PanelRight.Source = BitmapFrame.Create(Stream);
            }

            using (Stream Stream = ThisAssembly.GetManifestResourceStream(NameSpace+".Resources.PANEL_Center.png"))
            {
                PanelCenter.Source = BitmapFrame.Create(Stream);
            }

            BTN_Play = new ButtonGraphics(NameSpace+".Resources.PlayButton.bmp");
            BTN_Pause = new ButtonGraphics(NameSpace+".Resources.BTN_Pause.png");
            #endregion

            #region setup media player
            MainView.LoadedBehavior = MediaState.Manual;
            MainView.UnloadedBehavior = MediaState.Manual;
            MainView.MediaOpened += new RoutedEventHandler(MainView_MediaOpened);
            MainView.MediaEnded += new RoutedEventHandler(MainView_MediaEnded);
            MainView.Volume = 0.1;
            #endregion

            #region setup buttons
            PlayButton.Source = BTN_Play.GetImage(0);

            PlayButton.MouseEnter += new MouseEventHandler(PlayButton_MouseEnter);
            PlayButton.MouseLeave += new MouseEventHandler(PlayButton_MouseLeave);
            PlayButton.MouseDown += new MouseButtonEventHandler(PlayButton_MouseDown);
            PlayButton.MouseUp += new MouseButtonEventHandler(PlayButton_MouseUp);

            PauseButton.MouseEnter += new MouseEventHandler(PauseButton_MouseEnter);
            PauseButton.MouseLeave += new MouseEventHandler(PauseButton_MouseLeave);
            PauseButton.MouseDown += new MouseButtonEventHandler(PauseButton_MouseDown);
            PauseButton.MouseUp += new MouseButtonEventHandler(PauseButton_MouseUp);
            #endregion

            #region setup speech
            LoadXml();

            RebuildGrammar();

            Keyword.Script.OnSay += new Keyword.Script.ScriptDelegate(Script_OnSay);
            Keyword.Script.OnCommand += new Keyword.Script.ScriptDelegate(Script_OnCommand);

            Speech.SpeechEnabled = true;
            Speech.appRecognizer.SpeechHypothesized += new EventHandler<System.Speech.Recognition.SpeechHypothesizedEventArgs>(appRecognizer_SpeechHypothesized);
            Speech.appRecognizer.SpeechDetected += new EventHandler<System.Speech.Recognition.SpeechDetectedEventArgs>(appRecognizer_SpeechDetected);
            #endregion
        }

        #region handling speech feedback
        SpeechSynthesizer ttsSynth = new SpeechSynthesizer();
        LinkedList<String> LastStatements = new LinkedList<string>();

        void DisplayText(string text)
        {
            LastStatements.AddLast(text);
            if (LastStatements.Count > 4) LastStatements.RemoveFirst();
            SpeechText.Text = String.Join("\r\n", LastStatements.ToArray());
        }

        void Script_OnCommand(string say, Keyword.Script sender)
        {
            DisplayText("Executing: " + say);
            switch (say)
            {
                case "shutup":
                    ttsSynth.SpeakAsyncCancelAll();
                    break;
            }
        }

        void Script_OnSay(string say, Keyword.Script sender)
        {
            DisplayText("HAL: " + say);
            ttsSynth.SpeakAsync(say);
        }
        #endregion

        #region handling media player 

        void MainView_MediaEnded(object sender, RoutedEventArgs e)
        {
            CurrentPlayListEntry = CurrentPlayList.GetNextEntry(CurrentPlayListEntry);
        }

        void MainView_MediaOpened(object sender, RoutedEventArgs e)
        {
            MediaLength = MainView.NaturalDuration;
        }

        Duration MediaLength;
        Uri _CurrentTrack;
        Uri CurrentTrack
        {
            get
            {
                return _CurrentTrack;
            }
            set
            {
                if (_CurrentTrack != value)
                {
                    _CurrentTrack = value;
                    MainView.Source = value;
                    MainView.Play();
                }
            }

        }

        PlayList.PlayListEntry _CurrentPlayListEntry;
        PlayList.PlayListEntry CurrentPlayListEntry
        {
            get
            {
                return _CurrentPlayListEntry;
            }
            set
            {
                if (_CurrentPlayListEntry != value)
                {
                    _CurrentPlayListEntry = value;
                    if (_CurrentPlayListEntry != null)
                    {
                        if (!System.IO.Path.IsPathRooted(_CurrentPlayListEntry.Source))
                        {
                            String s = System.IO.Path.Combine(System.IO.Directory.GetCurrentDirectory(), _CurrentPlayListEntry.Source);
                            if (System.IO.File.Exists(s))
                            {
                                CurrentTrack = new Uri(s);
                            }
                        }
                        else
                        {
                            CurrentTrack = new Uri(_CurrentPlayListEntry.Source);
                        }
                    }
                    else
                    {
                        CurrentTrack = new Uri("");
                    }
                }
            }
        }

        PlayList _CurrentPlayList;
        PlayList CurrentPlayList
        {
            get
            {
                return _CurrentPlayList;
            }
            set
            {
                if (_CurrentPlayList != value)
                {
                    _CurrentPlayList = value;
                    if (value != null && _CurrentPlayListEntry != null && _CurrentPlayList.NamedTracks.Keys.Contains(_CurrentPlayListEntry.Name))
                    {
                        CurrentPlayListEntry = _CurrentPlayList[_CurrentPlayListEntry.Name];
                    }
                    else
                    {
                        CurrentPlayListEntry = _CurrentPlayList.Tracks.Keys.First();
                    }
                }
            }
        }

        #endregion

        #region Speech API debug events
        void appRecognizer_SpeechDetected(object sender, System.Speech.Recognition.SpeechDetectedEventArgs e)
        {
        }

        void appRecognizer_SpeechHypothesized(object sender, System.Speech.Recognition.SpeechHypothesizedEventArgs e)
        {
            Title = e.Result.Text;
            Debug.WriteLine("Speech Hypothesis: " + e.Result.Text);
        }
        #endregion

        #region Load XML

        Dictionary<String, Keyword> Keywords;
        Dictionary<String, PlayList> PlayLists;

        private void LoadXml()
        {
            XmlDocument doc = new XmlDocument();
            doc.Load("commands.xml");
            LoadPlayLists(doc.DocumentElement["playlists"]);
            LoadKeywords(doc.DocumentElement["keywords"]);
        }

        private void LoadKeywords(XmlElement xmlElement)
        {
            Keywords = new Dictionary<string, Keyword>();

            foreach (XmlElement playlist in xmlElement.GetElementsByTagName("keyword"))
            {
                Keyword kw = new Keyword();
                kw.Name = playlist.SelectSingleNode("name").InnerText;

                foreach (XmlElement track in playlist["scripts"].GetElementsByTagName("script"))
                {
                    Keyword.Script entry = new Keyword.Script();
                    entry.Name = track["name"].InnerText;
                    if (track.SelectSingleNode("action/url") != null) entry.Url = track.SelectSingleNode("action/url").InnerText;
                    if (track.SelectSingleNode("action/say") != null) entry.Say = track.SelectSingleNode("action/say").InnerText;
                    if (track.SelectSingleNode("action/initialsay") != null) entry.InitialSay = track.SelectSingleNode("action/initialsay").InnerText;
                    if (track.SelectSingleNode("action/failsay") != null) entry.FailSay = track.SelectSingleNode("action/failsay").InnerText;
                    if (track.SelectSingleNode("action/seek") != null) entry.Seek = track.SelectSingleNode("action/seek").InnerText;
                    if (track.SelectSingleNode("action/alternateseek") != null) entry.AlternateSeek = track.SelectSingleNode("action/alternateseek").InnerText;
                    if (track.SelectSingleNode("action/regex") != null) entry.sRegEx = track.SelectSingleNode("action/regex").InnerText;
                    if (track.SelectSingleNode("action/command") != null) entry.Command = track.SelectSingleNode("action/command").InnerText;

                    if (track.SelectSingleNode("action/queries") != null)
                    {
                        foreach (XmlElement query in track["action"]["queries"].GetElementsByTagName("query"))
                        {
                            Keyword.Script.RegularExpression regex = new Keyword.Script.RegularExpression();
                            if (query.SelectSingleNode("say") != null) regex.Say = query.SelectSingleNode("say").InnerText;
                            if (query.SelectSingleNode("seek") != null) regex.Seek = query.SelectSingleNode("seek").InnerText;
                            if (query.SelectSingleNode("regex") != null) regex.sRegEx = query.SelectSingleNode("regex").InnerText;
                            entry.Queries.Add(regex);
                        }
                    }

                    if (track.SelectSingleNode("pronounciations") != null)
                    {
                        foreach (XmlElement pronounciation in track["pronounciations"].GetElementsByTagName("pronounciation"))
                        {
                            entry.Pronounciations.Add(pronounciation.InnerText);
                        }
                    }

                    kw.Scripts.Add(entry);
                }

                if (playlist.SelectSingleNode("pronounciations") != null)
                {
                    foreach (XmlElement pronounciation in playlist["pronounciations"].GetElementsByTagName("pronounciation"))
                    {
                        kw.Pronounciations.Add(pronounciation.InnerText);
                    }
                }
                Keywords.Add(kw.Name, kw);
            }

        }

        private void LoadPlayLists(XmlElement xmlElement)
        {
            PlayLists = new Dictionary<string,PlayList>();

            foreach (XmlElement playlist in xmlElement.GetElementsByTagName("playlist"))
            {
                PlayList pl = new PlayList();
                pl.Name = playlist.SelectSingleNode("name").InnerText;

                foreach (XmlElement track in playlist["tracks"].GetElementsByTagName("track"))
                {
                    PlayList.PlayListEntry entry = new PlayList.PlayListEntry();
                    entry.Index = track["index"].InnerText;
                    entry.Name = track["name"].InnerText;
                    entry.Source = track["source"].InnerText;
                    pl.Tracks.Add(entry, entry);
                }

                if (playlist.SelectSingleNode("pronounciations") != null)
                {

                    foreach (XmlElement pronounciation in playlist["pronounciations"].GetElementsByTagName("pronounciation"))
                    {
                        pl.Pronounciations.Add(pronounciation.InnerText);
                    }
                }
                PlayLists.Add(pl.Name, pl);

            }
        }

        #endregion

        #region Xml Data -> Speech

        private void RebuildGrammar()
        {
            Speech = new SpeechEngine();
            foreach (Keyword kw in Keywords.Values)
            {
                SpeechElement SpeechKeyword = new SpeechElement(Speech, kw.Name);
                foreach (String Pronounciation in kw.Pronounciations)
                {
                    SpeechKeyword.AddPronounciation(Pronounciation);
                }


                foreach (Keyword.Script script in kw.Scripts)
                {
                    SpeechElement SpeechScript = new SpeechElement(SpeechKeyword, script.Name);
                    foreach (String Pronounciation in script.Pronounciations)
                    {
                        SpeechScript.AddPronounciation(Pronounciation);
                    }
                    SpeechScript.TagObject = script;

                    SpeechScript.OnMatch += new SpeechElement.SpeechMatch(delegate(SpeechElement match)
                    {
                        this.Dispatcher.BeginInvoke((System.Windows.Forms.MethodInvoker)delegate()
                        {
                            (match.TagObject as Keyword.Script).Execute();
                        });
                    });
                }
            }

            SpeechElement Play = new SpeechElement(Speech, "Play");
            foreach (PlayList pl in PlayLists.Values)
            {
                SpeechElement PlayListSpeech = new SpeechElement(Play, pl.Name);
                foreach (String Pronounciation in pl.Pronounciations)
                {
                    PlayListSpeech.AddPronounciation(Pronounciation);
                }
                PlayListSpeech.TagObject = pl;

                PlayListSpeech.OnMatch += new SpeechElement.SpeechMatch(delegate(SpeechElement match)
                {
                    this.Dispatcher.BeginInvoke((System.Windows.Forms.MethodInvoker)delegate()
                    {
                        CurrentPlayList = match.TagObject as PlayList;
                    });
                });
            }
        }

        #endregion

        #region GUI related events & Resources

        ButtonGraphics BTN_Play;
        ButtonGraphics BTN_Pause;

        void PauseButton_MouseUp(object sender, MouseButtonEventArgs e)
        {
            PlayButton.Visibility = Visibility.Visible;
            PlayButton.Source = BTN_Play.GetImage(4);
            PauseButton.Visibility = Visibility.Hidden;
        }

        void PauseButton_MouseDown(object sender, MouseButtonEventArgs e)
        {
            PauseButton.Source = BTN_Pause.GetImage(1);
            MainView.Pause();
        }

        void PauseButton_MouseLeave(object sender, MouseEventArgs e)
        {
            PauseButton.Source = BTN_Pause.GetImage(0);
        }

        void PauseButton_MouseEnter(object sender, MouseEventArgs e)
        {
            PauseButton.Source = BTN_Pause.GetImage(4);
        }

        void PlayButton_MouseUp(object sender, MouseButtonEventArgs e)
        {
            PauseButton.Source = BTN_Pause.GetImage(4);
            PauseButton.Visibility = Visibility.Visible;
            PlayButton.Visibility = Visibility.Hidden;
            PlayButton.IsEnabled = false;
        }

        void PlayButton_MouseDown(object sender, MouseButtonEventArgs e)
        {
            PlayButton.Source = BTN_Play.GetImage(1);
            if (CurrentPlayList == null)
            {
                if (PlayLists.Count == 0) return;
                CurrentPlayList = PlayLists.First().Value;
            }
            MainView.Play();
        }

        void PlayButton_MouseLeave(object sender, MouseEventArgs e)
        {
            PlayButton.Source = BTN_Play.GetImage(0);
        }

        void PlayButton_MouseEnter(object sender, MouseEventArgs e)
        {
            PlayButton.Source = BTN_Play.GetImage(4);
        }

        #endregion
    }
}
