﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Text.RegularExpressions;

namespace LabiHAL
{
    /// <summary>
    /// A LabiHAL Keyword is something that precedes a voice command.
    /// </summary>
    public class Keyword
    {
        /// <summary>
        /// The LabiHAL Keyword.Script is a full voice command.
        /// </summary>
        public class Script
        {
            /// <summary>
            /// The LabiHAL Keyword.Script.RegularExpression is a more complex way of handling commands.
            /// </summary>
            public class RegularExpression
            {
                public string Seek { get; set; }
                public string sRegEx { get; set; }
                public string Say { get; set; }
                public bool Match(string source, out string say)
                {
                    string[] args = new string[0];
                    say = null;
                    if (!String.IsNullOrEmpty(Seek))
                    {
                        int index = source.IndexOf(Seek);
                        if (index >= 0)
                        {
                            source = source.Substring(index);
                        }
                        else
                        {
                            return false;
                        }
                    }

                    if (!String.IsNullOrEmpty(sRegEx))
                    {
                        Regex regex = new Regex(sRegEx, RegexOptions.Compiled);
                        Match match = regex.Match(source.Replace('\n', ' ').Replace('\r', ' '));
                        if (match.Success)
                        {
                            List<String> argList = new List<string>();
                            foreach (Group g in match.Groups)
                            {
                                foreach (Capture capture in g.Captures)
                                {
                                    argList.Add(capture.Value);
                                }
                            }
                            args = argList.ToArray();
                        }
                        else
                        {
                            return false;
                        }
                    }

                    say = String.Format(Say, args);
                    return true;
                }
            }

            #region Script class
            public string Name { get; set; }
            public string Url { get; set; }
            public string Seek { get; set; }
            public string AlternateSeek { get; set; }
            public string sRegEx { get; set; }
            public string Say { get; set; }
            public string InitialSay { get; set; }
            public string FailSay { get; set; }
            public string Command { get; set; }


            private List<RegularExpression> _Queries = new List<RegularExpression>();
            public IList<RegularExpression> Queries { get { return _Queries; } }

            private List<String> _Pronounciations = new List<String>();
            public IList<String> Pronounciations { get { return _Pronounciations; } }

            public void Execute()
            {
                if (!String.IsNullOrEmpty(InitialSay))
                {
                    DoSay(InitialSay);
                }

                string[] args = new string[0];
                string web = null;
                if (!String.IsNullOrEmpty(Url))
                {
                    WebRequest wrGETURL;
                    wrGETURL = WebRequest.Create(Url);

                    wrGETURL.Proxy = WebRequest.DefaultWebProxy;

                    wrGETURL.Timeout = 10000;
                    try
                    {
                        using (Stream objStream = wrGETURL.GetResponse().GetResponseStream())
                        {
                            StreamReader objReader = new StreamReader(objStream);

                            web = objReader.ReadToEnd();
                        }
                    }
                    catch (System.Net.WebException)
                    {
                        return;
                    }
                }

                if (!String.IsNullOrEmpty(web) && Queries.Count != 0)
                {
                    string say;
                    foreach (RegularExpression reg in Queries)
                    {
                        if (reg.Match(web, out say))
                        {
                            DoSay(say);
                            return;
                        }
                    }
                }

                if (!String.IsNullOrEmpty(web) && !String.IsNullOrEmpty(Seek))
                {
                    int index = web.IndexOf(Seek);
                    if (index < 0 && !String.IsNullOrEmpty(AlternateSeek))
                    {
                        index = web.IndexOf(AlternateSeek);
                    }
                    if (index >= 0)
                    {
                        web = web.Substring(index);
                    }
                    else
                    {
                        web = "";
                    }
                }

                if (!String.IsNullOrEmpty(sRegEx))
                {
                    if (!String.IsNullOrEmpty(web))
                    {
                        Regex regex = new Regex(sRegEx, RegexOptions.Compiled);
                        Match match = regex.Match(web);
                        if (match.Success)
                        {
                            List<String> argList = new List<string>();
                            foreach (Group g in match.Groups)
                            {
                                foreach (Capture capture in g.Captures)
                                {
                                    argList.Add(capture.Value);
                                }
                            }
                            args = argList.ToArray();
                        }
                        else
                        {
                            if (!String.IsNullOrEmpty(FailSay))
                            {
                                DoSay(String.Format(FailSay, args));
                                return;
                            }
                        }
                    }
                    else
                    {
                        if (!String.IsNullOrEmpty(FailSay))
                        {
                            DoSay(String.Format(FailSay, args));
                            return;
                        }
                    }
                }

                if (!String.IsNullOrEmpty(Say))
                {
                    DoSay(String.Format(Say, args));
                }
                if (!String.IsNullOrEmpty(Command))
                {
                    DoCommand(String.Format(Command, args));
                }
            }

            public delegate void ScriptDelegate(string say, Script sender);
            public static event ScriptDelegate OnSay;
            public static event ScriptDelegate OnCommand;
            private void DoSay(string say)
            {
                if (OnSay != null)
                {
                    OnSay(say, this);
                }
            }
            private void DoCommand(string command)
            {
                if (OnCommand != null)
                {
                    OnCommand(command, this);
                }
            }
            #endregion
        }

        #region Keyword class
        public string Name { get; set; }
        private List<Script> _Scripts = new List<Script>();
        public ICollection<Script> Scripts { get { return _Scripts; } }

        private List<String> _Pronounciations = new List<String>();
        public IList<String> Pronounciations { get { return _Pronounciations; } }
        #endregion
    }
}
