﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace LabiHAL
{
    /// <summary>
    /// A LabiHAL PlayList is merely a list of PlayListEntry
    /// </summary>
    public class PlayList
    {
        /// <summary>
        /// A LabiHAL PlayList.PlayListEntry is merely a sortable data holder for Name and Source.
        /// </summary>
        public class PlayListEntry : IComparable<PlayListEntry>
        {
            public String Name { get; set; }
            public String Source { get; set; }
            public String Index { get; set; }

            #region IComparable<PlayListEntry> Members

            public int CompareTo(PlayListEntry other)
            {
                int v1;
                int v2;

                if (int.TryParse(other.Index, out v1) && int.TryParse(Index, out v2))
                {
                    if (v1 != v2)
                    {
                        return v2.CompareTo(v1);
                    }
                }
                else
                {
                    if (other.Index != Index) return Index.CompareTo(other.Index);
                }
                if (Name != other.Name) return Name.CompareTo(other.Name);
                return Source.CompareTo(other.Source);
            }

            #endregion
        }

        #region PlayList class
        public String Name { get; set; }
        private List<String> _Pronounciations = new List<String>();
        public IList<String> Pronounciations { get { return _Pronounciations; } }

        private SortedDictionary<PlayListEntry, PlayListEntry> _Tracks = new SortedDictionary<PlayListEntry, PlayListEntry>();
        public IDictionary<PlayListEntry, PlayListEntry> Tracks { get { return _Tracks; } }

        private Dictionary<String, PlayListEntry> _NamedTracks = new Dictionary<String, PlayListEntry>();
        public IDictionary<String, PlayListEntry> NamedTracks 
        {
            get
            {
                if (_NamedTracks.Count != _Tracks.Count)
                {
                    _Tracks.Clear();
                    foreach (PlayListEntry ple in _Tracks.Keys)
                    {
                        _NamedTracks.Add(ple.Name, ple);
                    }
                }
                return _NamedTracks;
            }
        }
        public PlayListEntry this[String name]
        {
            get
            {
                return NamedTracks[name];
            }
        }

        internal PlayListEntry GetNextEntry(PlayListEntry CurrentPlayListEntry)
        {
            bool found = false;
            foreach(PlayListEntry entry in _Tracks.Keys)
            {
                if (found) return entry;
                if (entry == CurrentPlayListEntry) found = true;
            }
            return _Tracks.Keys.First();
        }
        #endregion
    }
}
